/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "magnum.scene3d.h"
#include "graphics.h"
#include "models.h"
#include "gui.h"

#include "graphics.inl"
#include "gui.inl"


namespace magnum { namespace sample {
using namespace magnum::graphics;
using magnum::io::SystemFileFactory;
using magnum::geometry::Ray;
using magnum::graphics3d::StaticModel;
using magnum::graphics3d::PointLight;
using magnum::graphics3d::Context;
using magnum::graphics3d::DepthMode;
using magnum::graphics3d::PolygonMode;
using magnum::graphics3d::StencilMode;
using magnum::graphics3d::BlendMode;
using magnum::graphics3d::GeometryRenderer;
using magnum::scene3d::Universe;
using magnum::scene3d::Root;
using magnum::scene3d::Locale;
using magnum::scene3d::Viewer;
using magnum::scene3d::ModelNode;
using magnum::scene3d::PointLightNode;
using magnum::scene3d::Observer;
using magnum::scene3d::PickTraverser;
using magnum::scene3d::Graphics;
using magnum::scene3d::Intersection;


static SystemFileFactory	g_FileFactory("data");
static StaticModel*			g_Model;
static PointLight*			g_Light;
static StringView*			g_Status;
static Universe*			g_Universe;
static Locale*				g_Locale;
static Root*				g_Root;
static Observer*			g_Observer;
static Viewer*				g_Viewer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void insertModel(int x,int y) {
	Transform3<float> t = Transform3<float>::Identity;
	t.translate(Vector3<float>(320.0f*x,600.0f*y,0));
	AutoPointer<ModelNode> modelnode = new ModelNode(g_Model);
	modelnode->setTransform(t);
	g_Root->insert(modelnode);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onInit() {
	// Create Universe
	g_Universe = new Universe();
	g_Locale = new Locale(g_Universe);
	g_Root = new Root();
	g_Locale->insert(g_Root);

	g_Viewer = new Viewer();
	g_Viewer->setNearPlane(20);
	g_Viewer->setFarPlane(4000);
	g_Root->insert(g_Viewer);

	g_Observer = new Observer(g_Universe, m_Context, NULL);
	g_Observer->attachViewer(g_Viewer);

	g_Light = new PointLight();
	g_Light->setPosition(Vector3<float>(0.0f,0.0f,400.0f));
	g_Light->setAmbient(Colour<float>(0.0f,0.0f,0.0f));
	g_Light->setDiffuse(Colour<float>(1.0f,1.0f,1.0f));
	g_Light->setSpecular(Colour<float>(0.3f,0.3f,0.3f));

	g_Model = (StaticModel*)create3ds(&g_FileFactory, "f5e_05.3ds");
	g_Model->optimise();
	g_Model->updateCollider();
	
	for (int i = -3; i <= 3; ++i) {
		for (int j = -2; j <= 2; ++j) {
			insertModel(i,j);	
		}
	}
	
	g_Status = new StringView(Rectangle<float>(10,480,170,40), "");
	g_Status->setAlignement(View::FollowBottom);
	g_Status->setFont(m_Gui->getFont());
	getGui()->insert(g_Status);

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onExit() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onHandleMessage(const Message& msg) {
	return OpenGL::onHandleMessage(msg);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GraphicsView::onRender() {
	DepthMode::selectDefault();
	BlendMode::selectDefault();
	
	m_Context->setClearColour(Colour<float>(0.59f, 0.59f, 1.0f));
	m_Context->clear(Context::ClearColour | Context::ClearDepth | Context::ClearStencil);

	m_MatrixStack->selectMode(MatrixStack::ModelView);
	m_MatrixStack->identity();
	g_Light->select(0);

	g_Viewer->setTransform(getTransform());
	g_Universe->update();
	g_Observer->view();
	
	float width = getBounds().getWidth();
	float height = getBounds().getHeight();
	GeometryRenderer::renderAxes(getTransform(), Vector2<float>(width - 32, height - 32));
	m_Context->renderDefered();
	
	PickTraverser<Graphics> picker(g_Universe);
	Intersection is;
	Point<float> mouse = m_Gui->getMouse();
	float w2 = 0.5f*getBounds().getWidth();
	float h2 = 0.5f*getBounds().getHeight();
	Ray<float,3> ray = g_Viewer->createRay((mouse.x - w2) / w2, (mouse.y - h2) / h2);
//	printf("ray dir = %f, %f, %f\n", ray.getDirection()[0],ray.getDirection()[1],ray.getDirection()[2]);
//	Ray<float,3> ray = g_Viewer->createRay(0,0);
	if (picker.traverse(ray, &is))
		g_Status->setText("mouse over object");
	else
		g_Status->setText("mouse not over object");

	m_Gui->process();
	m_Gui->paint();
}

}; };

