/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "models.h"

namespace magnum { namespace sample {
using namespace magnum::graphics;
using namespace magnum::graphics3d;
using namespace magnum::io;

static void printBounds(Model* model) {
	Bounds3<float> bounds = model->getBounds();
	printf("bounds: origin = (%f,%f,%f)\n", bounds.getOrigin()[0], bounds.getOrigin()[1], bounds.getOrigin()[2]);
	printf("       axis[0] = (%f,%f,%f)\n",	bounds.getAxis(0)[0], bounds.getAxis(0)[1], bounds.getAxis(0)[2]);
	printf("       axis[1] = (%f,%f,%f)\n",	bounds.getAxis(1)[0], bounds.getAxis(1)[1], bounds.getAxis(1)[2]);
	printf("       axis[2] = (%f,%f,%f)\n",	bounds.getAxis(2)[0], bounds.getAxis(2)[1], bounds.getAxis(2)[2]);
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StaticModel* create3ds(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	Translator3DS t;
	t.attachTextureManager(&mgr);

	StaticModel* model = static_cast<StaticModel*>(t.decode(file->getInputStream()));
//	printBounds(model);
	
	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FrameModel* createMd2(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorMD2 t;
	t.attachTextureManager(&mgr);

	FrameModel* model = static_cast<FrameModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
FrameModel* createMd3(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorMD3 t;
	t.attachTextureManager(&mgr);

	FrameModel* model =  static_cast<FrameModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
BoneModel* createMds(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorMDS t;
	t.attachTextureManager(&mgr);

	BoneModel* model = static_cast<BoneModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StaticModel* createObj(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorOBJ t;
	t.attachTextureManager(&mgr);

	StaticModel* model = static_cast<StaticModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StaticModel* createIfs(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorIFS t;
	t.attachTextureManager(&mgr);

	StaticModel* model = static_cast<StaticModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StaticModel* createPly(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorPLY t;
	t.attachTextureManager(&mgr);

	StaticModel* model = static_cast<StaticModel*>(t.decode(file->getInputStream()));

	return model;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StaticModel* createLwo(FileFactory* ff,const char* fname) {
	AutoPointer<File> file = ff->open(fname);
	TextureManager mgr;
	mgr.getFactory()->setFileFactory(ff);

	TranslatorLWO t;
	t.attachTextureManager(&mgr);

	StaticModel* model = static_cast<StaticModel*>(t.decode(file->getInputStream()));

	return model;
}


}; };
